package cz.muni.fi.pv168.gravemanager.web;

import cz.muni.fi.pv168.common.DBUtils;
import cz.muni.fi.pv168.gravemanager.backend.Grave;
import cz.muni.fi.pv168.gravemanager.backend.GraveManager;
import cz.muni.fi.pv168.gravemanager.backend.GraveManagerImpl;
import java.io.IOException;
import java.sql.SQLException;
import javax.annotation.Resource;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

@WebServlet(
        name = "ActionServlet", 
        urlPatterns = {
            ActionServlet.ACTION_LIST_GRAVES, 
            ActionServlet.ACTION_ADD_GRAVE})
public class ActionServlet extends HttpServlet {

    static final String ACTION_ADD_GRAVE = "/AddGrave";
    static final String ACTION_LIST_GRAVES = "/ListGraves";    
    static final String ATTRIBUTE_GRAVES = "graves";    
    static final String ATTRIBUTE_GRAVE_FORM = "graveForm";
    static final String ATTRIBUTE_ERROR = "error";
    static final String JSP_ADD_GRAVE = "/WEB-INF/jsp/addGrave.jsp";
    static final String JSP_LIST_GRAVES = "/WEB-INF/jsp/listGraves.jsp";

    private GraveManagerImpl graveManager = new GraveManagerImpl();

    @Resource(name="jdbc/GraveManager")
    private void setDataSource(DataSource ds) throws SQLException {
        DBUtils.tryCreateTables(ds,GraveManager.class.getResource("createTables.sql"));
        graveManager.setDataSource(ds);
    }

    private void listGraves(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
        request.setAttribute(ATTRIBUTE_GRAVES, graveManager.findAllGraves());
        request.getRequestDispatcher(JSP_LIST_GRAVES).forward(request, response);
    }
        
    private void addGrave(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

        if (request.getMethod().equals("POST")) {
            
            if (request.getParameter("cancel") != null) {
                response.sendRedirect(request.getContextPath());
                return;
            }
            
            GraveForm graveForm = GraveForm.extractFromRequest(request);

            StringBuilder errors = new StringBuilder();
            Grave grave = graveForm.validateAndToGrave(errors);

            if (grave == null) {
                request.setAttribute(ATTRIBUTE_ERROR, errors.toString());
                request.setAttribute(ATTRIBUTE_GRAVE_FORM, graveForm);
                request.getRequestDispatcher(JSP_ADD_GRAVE).forward(request, response);
            } else {
                graveManager.createGrave(grave);
                response.sendRedirect(request.getContextPath());
            }

        } else {
            request.setAttribute(ATTRIBUTE_GRAVE_FORM, new GraveForm());
            request.getRequestDispatcher(JSP_ADD_GRAVE).forward(request, response);
        }
    }
        
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        response.setContentType("text/html;charset=UTF-8");
        
        if (request.getServletPath().equals(ACTION_LIST_GRAVES)) {
            listGraves(request, response);
        } else if (request.getServletPath().equals(ACTION_ADD_GRAVE)) {
            addGrave(request, response);
        } else {
            throw new RuntimeException("Unknown operation: " + request.getServletPath());
        }        
    }

    // <editor-fold defaultstate="collapsed" desc="HttpServlet methods. Click on the + sign on the left to edit the code.">
    /**
     * Handles the HTTP
     * <code>GET</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Handles the HTTP
     * <code>POST</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Returns a short description of the servlet.
     *
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>
}
