package cz.muni.fi.pv168.gravemanager.web;

import cz.muni.fi.pv168.gravemanager.backend.Grave;
import javax.servlet.http.HttpServletRequest;

/**
 * This class holds data entered into Grave web form.
 * 
 * @author Petr Adamek
 */
public class GraveForm {

    private Long id;
    private String column;
    private String row;
    private String capacity;
    private String note;

    public static GraveForm extractFromRequest(HttpServletRequest request) {
        GraveForm graveForm = new GraveForm();
        String idString = request.getParameter("id");        
        graveForm.setId(idString == null?null:Long.parseLong(idString));
        graveForm.setCapacity(request.getParameter("capacity"));
        graveForm.setColumn(request.getParameter("column"));
        graveForm.setRow(request.getParameter("row"));
        graveForm.setNote(request.getParameter("note"));
        return graveForm;
    }
    
    private static int stringToInt(String value, String fieldName, int minValue, StringBuilder errors) {
        if (value == null || value.isEmpty()) {
            errors.append("Field '").append(fieldName).append("' is not filled").append(". <br />");
            return 0;
        }
        int result;
        try {
            result = Integer.parseInt(value);
        } catch (NumberFormatException ex) {
            errors.append("Field '").append(fieldName).append("' is not a number: ").append(value).append(". <br />");
            return 0;
        }        
        if (result < minValue) {
            errors.append("Field '").append(fieldName).append("' has illegal value: ").append(result).append(". <br />");
            return 0;            
        }
        return result;
    }
    
    public Grave validateAndToGrave(StringBuilder errors) {
        Grave grave = new Grave();
        grave.setCapacity(stringToInt(getCapacity(), "capacity", 1, errors));
        grave.setRow(stringToInt(getRow(), "row", 0, errors));
        grave.setColumn(stringToInt(getColumn(), "column", 0, errors));
        if (getNote() == null || getNote().isEmpty()) {
            grave.setNote(null);            
        } else {
            grave.setNote(getNote());                                    
        }
        if (errors.length() > 0) {
            return null;
        } else {
            return grave;
        }
    }
    
    public String getCapacity() {
        return capacity;
    }

    public void setCapacity(String capacity) {
        this.capacity = capacity;
    }

    public String getColumn() {
        return column;
    }

    public void setColumn(String column) {
        this.column = column;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getNote() {
        return note;
    }

    public void setNote(String note) {
        this.note = note;
    }

    public String getRow() {
        return row;
    }

    public void setRow(String row) {
        this.row = row;
    }    
    
}
